class UnrealShare_Rocket expands Rocket;

var() bool bAlwaysSynchronizeClient;

// replicated built-in vectors hold inexact values, float values are replicated precisely
struct ReplicatedVector
{
	var float X, Y, Z;
};

var float InitialSpeed;

var bool bSynchronizeClient;
var ReplicatedVector ReplicatedLocation;
var ReplicatedVector ReplicatedVelocity;
var ReplicatedVector ReplicatedAcceleration;

var UGoldFixSharedInfo SharedInfo;

replication
{
	reliable if (Role == ROLE_Authority)
		InitialSpeed,
		ReplicatedLocation,
		ReplicatedVelocity,
		ReplicatedAcceleration;
}


simulated function PostBeginPlay()
{
	Super.PostBeginPlay();

	if (Level.NetMode == NM_Client && bAlwaysSynchronizeClient)
		DrawType = DT_None;
}

simulated function Tick(float DeltaTime)
{
	if (!Region.Zone.bWaterZone && VSize(Velocity) < InitialSpeed)
	{
		Velocity = Normal(Velocity) * InitialSpeed;
		Acceleration = Vector(Rotation) * 50;
	}

	if (bAlwaysSynchronizeClient || bSynchronizeClient)
		Server_UpdateMovement();
	bSynchronizeClient = false;

	Client_UpdateMovement();

	if (Level.NetMode == NM_DedicatedServer)
		return;

	MakeRocketTrail(DeltaTime);
}

simulated function MakeRocketTrail(float DeltaTime)
{
	local SpriteSmokePuff smoke;
	local Bubble1 bubble;
	local float RocketTrailInterval;

	Count += DeltaTime;
	RocketTrailInterval = SmokeRate + FRand() * (SmokeRate + NumExtraRockets * 0.035);
	if (Count >= RocketTrailInterval)
	{
		if (Region.Zone.bWaterZone)
		{
	 		bubble = Spawn(class'Bubble1',,, Location);
			if (bubble != none)
			{
				bubble.DrawScale = 0.1 + FRand() * 0.2;
				bubble.RemoteRole = ROLE_None;
			}
		}
		else
		{
			smoke = Spawn(class'SpriteSmokePuff');
			if (smoke != none)
				smoke.RemoteRole = ROLE_None;
		}
		Count -= RocketTrailInterval;
		if (Count > RocketTrailInterval)
			Count = RocketTrailInterval;
	}
}

auto state Flying
{
	simulated function ZoneChange(Zoneinfo NewZone)
	{
		local WaterRing w;

		if (VSize(NewZone.ZoneVelocity) != 0)
		{
			if (Level.NetMode == NM_Client)
				return;
			bSynchronizeClient = true;
		}

		if (Region.Zone.bWaterZone != NewZone.bWaterZone)
		{
			if (Level.NetMode != NM_Client)
			{
				w = Spawn(class'WaterRing',,,,rot(16384,0,0));
				w.DrawScale = 0.2;
			}

			if (NewZone.bWaterZone)
				Velocity = 0.6 * Velocity;
		}
		if (bSynchronizeClient)
			Server_UpdateMovement();
	}

	function ProcessTouch (Actor Other, Vector HitLocation)
	{
		if ((Other != instigator) && (Rocket(Other) == none))
			Explode(HitLocation,Normal(HitLocation-Other.Location));
	}

	function HitWall(vector HitNormal, actor Wall)
	{
		if (Mover(Wall) != None && Mover(Wall).bDamageTriggered)
			Wall.TakeDamage(Damage, instigator, Location, MomentumTransfer * Normal(Velocity), '');
		MakeNoise(1.0);
		Explode(Location + ExploWallOut * HitNormal, HitNormal);
	}

	function RocketBlowUp(vector HitLocation, bool bRingExplosion)
	{
		if (Level.Game.IsA('DeathMatchGame')) // bigger damage radius
			class'UGoldFix'.static.Projectile_HurtRadius(
				SharedInfo, self, 0.9 * Damage, 240.0, 'exploded', MomentumTransfer, HitLocation);
		else
			class'UGoldFix'.static.Projectile_HurtRadius(
				SharedInfo, self, Damage, 200.0, 'exploded', MomentumTransfer, HitLocation);
		MakeNoise(1.0);

		if (bRingExplosion)
			PlaySound(class'RingExplosion3'.default.ExploSound,, 6);
	}

	function Explode(vector HitLocation, vector HitNormal)
	{
		local SpriteBallExplosion explosion_effect;
		local UnrealShare_Rocket_Explosion explosion_info;

		if (Level.NetMode == NM_DedicatedServer)
		{
			// makes explosion sound on dedicated servers
			explosion_effect = Spawn(class'SpriteBallExplosion',,, HitLocation + HitNormal * 16);
			explosion_effect.RemoteRole = ROLE_None;
		}

		explosion_info = Spawn(class'UnrealShare_Rocket_Explosion');
		explosion_info.AssignSourceLocation(Location);
		explosion_info.AssignHitLocation(HitLocation);
		explosion_info.AssignHitNormal(HitNormal);
		explosion_info.SharedInfo = SharedInfo;
		explosion_info.bRing = bRing;

		RocketBlowUp(HitLocation, bRing);

		Destroy();
	}

	function BeginState()
	{
		SetTimer(0.15, True);

		initialDir = vector(Rotation);
		Velocity = Speed * initialDir;
		InitialSpeed = speed;
		Acceleration = initialDir * 50;
		PlaySound(SpawnSound, SLOT_None, 2.3);
		PlayAnim('Armed', 0.2);
		if (Region.Zone.bWaterZone)
			Velocity = 0.6 * Velocity;

		if (bAlwaysSynchronizeClient)
			Server_UpdateMovement();
	}
Begin:
	if (LifeSpan >= 2)
	{
		Sleep(1);
		LifeSpan += 1;
		Sleep(LifeSpan - 1);
		Explode(Location, vector(Rotation));
	}
}


function Timer()
{
	local vector SeekingDir;

	if (Seeking != None && Seeking != Instigator)
	{
		SeekingDir = Normal(Seeking.Location - Location);
		if (SeekingDir Dot InitialDir > 0)
		{
			MagnitudeVel = VSize(Velocity);
			Velocity =  MagnitudeVel * Normal(SeekingDir * 0.47 * MagnitudeVel + Velocity);
			SetRotation(rotator(Velocity));
		}
	}

	if (bAlwaysSynchronizeClient)
		Server_UpdateMovement();
}

function Server_UpdateMovement()
{
	if (Level.NetMode != NM_Standalone)
	{
		ReplicatedLocation = ConvertToReplicatedVector(Location);
		ReplicatedVelocity = ConvertToReplicatedVector(Velocity);
		ReplicatedAcceleration = ConvertToReplicatedVector(Acceleration);
	}
}

simulated function Client_UpdateMovement()
{
	if (Level.NetMode == NM_Client)
	{
		if (VSize(ConvertReplicatedVector(ReplicatedLocation)) != 0)
		{
			DrawType = DT_Mesh;
			SetLocation(ConvertReplicatedVector(ReplicatedLocation));
			ReplicatedLocation = ConvertToReplicatedVector(vect(0, 0, 0));
		}
		if (VSize(ConvertReplicatedVector(ReplicatedVelocity)) != 0)
		{
			Velocity = ConvertReplicatedVector(ReplicatedVelocity);
			Acceleration = ConvertReplicatedVector(ReplicatedAcceleration);
			SetRotation(rotator(Velocity));
			ReplicatedVelocity = ConvertToReplicatedVector(vect(0, 0, 0));
		}
	}
}

static function ReplicatedVector ConvertToReplicatedVector(vector pos)
{
	local ReplicatedVector result;
	result.X = pos.X;
	result.Y = pos.Y;
	result.Z = pos.Z;
	return result;
}

static function vector ConvertReplicatedVector(ReplicatedVector pos)
{
	local vector result;
	result.X = pos.X;
	result.Y = pos.Y;
	result.Z = pos.Z;
	return result;
}

defaultproperties
{
				bNetTemporary=False
				bGameRelevant=False
}
